Module: RubySMB::SMB1::CreateActions

Defined in:
lib/ruby_smb/smb1/oplock_levels.rb,
lib/ruby_smb/smb1/create_actions.rb

Overview

This module holds the Create Actions used in NT_TRANSACT_CREATE and SMB_COM_NT_CREATE_ANDX responses. The definitions for these values can be found at 2.2.7.1.2 Response

Constant Summary collapse

NO_OPLOCK =

No OpLock Granted

0x00
EXCLUSIVE_OPLOCK =

Exclusive OpLock Granted

0x01
BATCH_OPLOCK =

Batch OpLock Granted

0x02
LEVEL2_OPLOCK =

Level 2 OpLock Granted

0x03
FILE_SUPERSEDED =

An existing file was deleted and a new file was created in its place.

0x00000000
FILE_OPENED =

An existing file was opened.

0x00000001
FILE_CREATED =

A new file was created.

0x00000002
FILE_OVERWRITEN =

An existing file was overwritten.

0x00000003