Class: RubySMB::SMB1::Packet::Trans2::SetFsInformationRequestTrans2Parameters

Inherits:
BinData::Record
  • Object
show all
Defined in:
lib/ruby_smb/smb1/packet/trans2/set_fs_information_request.rb

Overview

The Trans2 Parameter Block for TRANS2_SET_FS_INFORMATION. Observed on the wire (and required by Samba) as a 4-byte block containing a placeholder file handle plus the information level.

The parent subcommand MS-CIFS 2.2.6.5 TRANS2_SET_FS_INFORMATION (0x0004) is marked "reserved but not implemented"; the on-the-wire format used here is defined by the CIFS UNIX Extensions draft and implemented in source3/smbd/smb1_trans2.c:1706-1915 (call_trans2setfsinfo).

Instance Method Summary collapse

Instance Method Details

#lengthObject

Returns the length of the Trans2Parameters struct in number of bytes



23
24
25
# File 'lib/ruby_smb/smb1/packet/trans2/set_fs_information_request.rb', line 23

def length
  do_num_bytes
end