Method: RubySMB::SMB2::File#set_header_fields
- Defined in:
- lib/ruby_smb/smb2/file.rb
#set_header_fields(request) ⇒ RubySMB::GenericPacket
Sets the header fields that we have to set on every packet we send for File operations.
229 230 231 232 233 |
# File 'lib/ruby_smb/smb2/file.rb', line 229 def set_header_fields(request) request = tree.set_header_fields(request) request.file_id = guid request end |